c /ngslib/source/Geoid/Flib/SCCS/s.bilin.f

      subroutine bilin(latdd,londd,lin,kk, val)
c ------------------------------------------------------------------------------
c Bilinear Interpolation  to find a value (val) from the kkth geoid grid. 
c As the gridded data are all direct-access files,
c only the nearest points (1 thru 9,
c depending on the point's location relative to corners and edges)
c are read into RAM for each point's interpolation.
c 
c The size/spacing/etc of the data file's grid
c are defined by the common statement, and
c the variable "kk"
c Caveats:
c   1) It is assumed that latdd/londd fall in the region of the kth grid
c   in - latdd  : North latitude  position, units = decdeg
c   in - londd  : East  longitude position, units = decdeg
c   in - lin    : vector of file unit numbers, pointing to grid files
c   in - kk     : grid file unit number 
c   out- val    : interpolated geoid height value, units as per data file
c ------------------------------------------------------------------------------
      implicit none
      integer      DEBUG
      parameter   (DEBUG = 0)                    ! diagnostics print if != 0

      character*90 SCCSID
      real*8     latdd
      real*8     londd
      real*8     glamn(50),glamx(50)
      real*8     glomn(50),glomx(50)
      real*8     dla(50),dlo(50)

      real*8     row_counter,col_counter

      real*4     xx, yy
      real*4     onemxx, onemyy
      real*4     UL, UR, LL, LR
      real*4     val
      real*4     row_val, col_val

      integer*4  kk
      integer*4  lin(50)
      integer*4  nla(50)
      integer*4  nlo(50)
      integer*4  ikind(50)
      integer    row
      integer    col

      real*8     latMin                        ! header record data
      real*8     lonMin
      real*8     latDelta
      real*8     lonDelta
      integer*4  latRowNum
      integer*4  lonColNum
      integer*4  irec
      real*8     latMax                        ! calcd from header data
      real*8     lonMax                        ! calcd from header data

c variables and functions for handling endian problem
      real*4     frev4
      logical    reverse_bytes(50)
      common     /endian/reverse_bytes

      common     /grid1/ glamn,glamx,glomn,glomx
      common     /grid2/ dla,dlo,nla,nlo,ikind

      SCCSID="@(#)bilin.f	1.10	2008/06/12 NGS"


c --------------------------------------------------------------------
c Define some necessary parameters
c These header data elements are read in the main driver file,
c are in a common block, and are already checked for endian condition
c --------------------------------------------------------------------
      latMin    = glamn(kk)
      lonMin    = glomn(kk)
      latDelta  = dla(kk)
      lonDelta  = dlo(kk)
      latRowNum = nla(kk)
      lonColNum = nlo(kk)
      latMax    = latMin + latDelta * (latRowNum - 1)
      lonMax    = lonMin + lonDelta * (lonColNum - 1)

      if (DEBUG .ge. 5) then
        print *, 'latMin      = ',latMin
        print *, 'lonMin      = ',lonMin
        print *, 'latDelta    = ',latDelta
        print *, 'lonDelta    = ',lonDelta
        print *, 'latRowNum   = ',latRowNum
        print *, 'lonColNum   = ',lonColNum
        print *, 'latMax      = ',latMax
        print *, 'lonMax      = ',lonMax
      endif

c --------------------------------------------------------------------
c Find the row/col of the nearest grid node to the lat/lon point
c This grid node is southwest from the lat/lon point
c (Row_counter,Col_counter) = exact (lat,lon) grid coord location (real*8)
c (Row,Col)                 = closest node (to sw) grid coord location (int)
c --------------------------------------------------------------------
      col_counter = ((londd-lonMin) / lonDelta) + 1   ! =fortran index corr'n
      row_counter = ((latdd-latMin) / latDelta) + 1   ! =fortran index corr'n

      row = int( row_counter + 0.00001 )    ! the row just south from latdd
      col = int( col_counter + 0.00001 )    ! the col just west  from londd

      row_val = (row-1)*latDelta + latMin   ! lat just south(up) from latdd
      col_val = (col-1)*lonDelta + lonMin   ! lon just west      from londd

C  Find relative location of the point in the interpolation window
      yy = (row_counter - row)              ! 1x1 window
      xx = (col_counter - col)

      onemyy = 1.0 - yy
      onemxx = 1.0 - xx

      if (DEBUG .gt. 2) then
        print *, 'in bilin: ---------------------------'
        print *, 'latMin      = ',latMin
        print *, 'latMax      = ',latMax
        print *, 'lonMin      = ',lonMin
        print *, 'lonMax      = ',lonMax
        print *, 'row_counter = ',row_counter
        print *, 'col_counter = ',col_counter
        print *, 'row         = ',row
        print *, 'col         = ',col
        print *, 'xx          = ',xx
        print *, 'yy          = ',yy
        print *, 'latdd       = ',latdd
        print *, 'londd       = ',londd
        print *, 'row_val     = ',row_val,
     +           '  (Nlat) should be: row_val < latdd'
        print *, 'col_val     = ',col_val,
     +           '  (Elon) should be: col_val < londd'
      endif

c ------------------------------------------------
c Extract 2x2 data array around the point 
c and do bilinear interpolation
c ------------------------------------------------
c          (col)     ---->        (col+1)
c
c (row-1)  LL ------------------------  LR
c             |     xx        | 1mxx |
c    |        | yy            |      |
c    |        |               |      |
c    |        |               |      |
c    |        |               |      |
c    \/       |----------------      |
c             | 1myy                 |
c             |                      |
c (row)    UL ------------------------  UR
c

C If point LL in on northern edge, then UL and UR are outside of dataset
C Linear interpolation on or near northern edge only (1*e-4 = 0.3 arcsec)

      if ((latMax - latdd) .lt. 0.0001) then
        irec = 11 + (row-1)*nlo(kk) + col
        read(lin(kk),rec=irec)LL

        irec = 11 + (row-1)*nlo(kk) + col + 1
        read(lin(kk),rec=irec)LR

        if(reverse_bytes(kk)) then
          LL=frev4(LL)
          LR=frev4(LR)
        endif

        val = (LL * onemxx / (xx + onemxx) )
     1      + (LR * xx     / (xx + onemxx) )
        return
      endif

C Bilinear interpolation
      irec = 11 + (row-1)*nlo(kk) + col
      read(lin(kk),rec=irec)LL

      irec = 11 + (row-1)*nlo(kk) + col + 1
      read(lin(kk),rec=irec)LR

      irec = 11 + (row  )*nlo(kk) + col
      read(lin(kk),rec=irec)UL

      irec = 11 + (row  )*nlo(kk) + col + 1
      read(lin(kk),rec=irec)UR

c convert big endian values to little endian
      if(reverse_bytes(kk)) then
        LL=frev4(LL)
        LR=frev4(LR)
        UL=frev4(UL)
        UR=frev4(UR)
      endif

      val = LL * onemyy * onemxx 
     1    + LR * xx     * onemyy 
     2    + UL * onemxx * yy
     3    + UR * xx     * yy

      return
      end
